@@ -801,7 +801,7 @@ static int mcontains(struct mpos pos,
 	struct elmnt e;
 	int found = 0;
 	struct mp mp;
-	int o;
+	int o = 0;
 	int len = strlen(search);
 
 	do {
@@ -1292,7 +1292,7 @@ static void *memdup(void *a, int len)
 }
 
 static int merge_window(struct plist *p, FILE *f, int reverse, int replace,
-			int selftest, int ignore_blanks, int just_diff)
+			int selftest, int ignore_blanks, int just_diff, int backup)
 {
 	/* Display the merge window in one of the selectable modes,
 	 * starting with the 'merge' mode.
@@ -1680,8 +1680,8 @@ static int merge_window(struct plist *p, FILE *f, int reverse, int replace,
 			attrset(A_NORMAL);
 		}
 		if (num >= 0) {
-			char buf[10];
-			snprintf(buf, 10, "%d ", num);
+			char buf[12+1];
+			snprintf(buf, sizeof(buf), "%d ", num);
 			addstr(buf);
 		}
 		if (meta & META(0))
@@ -1790,8 +1790,8 @@ static int merge_window(struct plist *p, FILE *f, int reverse, int replace,
 					ci.merger, 0, &p->wiggles);
 				p->chunks = p->conflicts;
 				save_merge(fm, fb, fa, ci.merger,
-					   p->outfile ?: p->file,
-					   p->outfile ? 0 : !p->is_merge);
+					   p->outfile ? p->outfile : p->file,
+					   backup && (p->outfile ? 0 : !p->is_merge));
 			}
 			if (!just_diff)
 				free(sm.body);
@@ -2347,7 +2347,7 @@ static int show_merge(char *origname, FILE *patch, int reverse,
 		      int is_merge, char *before, char *after,
 		      int replace, char *outfile,
 		      int selftest, int ignore_blanks,
-		      int just_diff)
+		      int just_diff, int backup)
 {
 	struct plist p = {0};
 
@@ -2366,7 +2366,7 @@ static int show_merge(char *origname, FILE *patch, int reverse,
 
 	freopen("/dev/null","w",stderr);
 	return merge_window(&p, patch, reverse, replace, selftest,
-			    ignore_blanks, just_diff);
+			    ignore_blanks, just_diff, backup);
 }
 
 static void calc_one(struct plist *pl, FILE *f, int reverse,
@@ -2427,7 +2427,7 @@ static void calc_one(struct plist *pl, FILE *f, int reverse,
 static int get_prev(int pos, struct plist *pl, int n, int mode)
 {
 	int found = 0;
-	if (pos == -1)
+	if (pos == -1 || pl == NULL)
 		return pos;
 	do {
 		if (pl[pos].prev == -1)
@@ -2487,7 +2487,7 @@ static int get_next(int pos, struct plist *pl, int n, int mode,
 static void draw_one(int row, struct plist *pl, FILE *f, int reverse,
 		     int ignore_blanks, int just_diff)
 {
-	char hdr[12];
+	char hdr[2*12];
 	hdr[0] = 0;
 
 	if (pl == NULL) {
@@ -2538,7 +2538,7 @@ static void draw_one(int row, struct plist *pl, FILE *f, int reverse,
 }
 
 static int save_one(FILE *f, struct plist *pl, int reverse,
-		    int ignore_blanks)
+		    int ignore_blanks, int backup)
 {
 	struct stream sp, sa, sb, sm;
 	struct file fa, fb, fm;
@@ -2559,7 +2559,7 @@ static int save_one(FILE *f, struct plist *pl, int reverse,
 	csl2 = diff_patch(fb, fa);
 	ci = make_merger(fm, fb, fa, csl1, csl2, 0, 1, 0);
 	return save_merge(fm, fb, fa, ci.merger,
-			  pl->file, 1);
+			  pl->file, backup);
 }
 
 static char *main_help[] = {
@@ -2613,8 +2613,8 @@ static char *saveall_query[] = {
 	"  Q = Don't quit just yet",
 	NULL
 };
-static void main_window(struct plist *pl, int *np, FILE *f, int reverse,
-			int replace, int ignore_blanks, int just_diff)
+static void main_window(struct plist *pl, int np, FILE *f, int reverse,
+			int replace, int ignore_blanks, int just_diff, int backup)
 {
 	/* The main window lists all files together with summary information:
 	 * number of chunks, number of wiggles, number of conflicts.
@@ -2669,7 +2669,6 @@ static void main_window(struct plist *pl, int *np, FILE *f, int reverse,
 
 	freopen("/dev/null","w",stderr);
 	term_init(1);
-	pl = sort_patches(pl, np);
 
 	while (1) {
 		if (refresh == 2) {
@@ -2694,7 +2693,7 @@ static void main_window(struct plist *pl, int *np, FILE *f, int reverse,
 				row = rows-1;
 			tpos = pos;
 			for (i = row; i > 1; i--) {
-				tpos = get_prev(tpos, pl, *np, mode);
+				tpos = get_prev(tpos, pl, np, mode);
 				if (tpos == -1) {
 					row = row - i + 1;
 					break;
@@ -2704,11 +2703,11 @@ static void main_window(struct plist *pl, int *np, FILE *f, int reverse,
 			tpos = pos;
 			for (i = row; i >= 1; i--) {
 				draw_one(i, &pl[tpos], f, reverse, ignore_blanks, just_diff);
-				tpos = get_prev(tpos, pl, *np, mode);
+				tpos = get_prev(tpos, pl, np, mode);
 			}
 			tpos = pos;
 			for (i = row+1; i < rows; i++) {
-				tpos = get_next(tpos, pl, *np, mode, f, reverse,ignore_blanks, just_diff);
+				tpos = get_next(tpos, pl, np, mode, f, reverse,ignore_blanks, just_diff);
 				if (tpos >= 0)
 					draw_one(i, &pl[tpos], f, reverse, ignore_blanks, just_diff);
 				else
@@ -2742,7 +2741,7 @@ static void main_window(struct plist *pl, int *np, FILE *f, int reverse,
 		case 'N':
 		case 'N'-64:
 		case KEY_DOWN:
-			tpos = get_next(pos, pl, *np, mode, f, reverse, ignore_blanks, just_diff);
+			tpos = get_next(pos, pl, np, mode, f, reverse, ignore_blanks, just_diff);
 			if (tpos >= 0) {
 				pos = tpos;
 				row++;
@@ -2753,7 +2752,7 @@ static void main_window(struct plist *pl, int *np, FILE *f, int reverse,
 		case 'P':
 		case 'P'-64:
 		case KEY_UP:
-			tpos = get_prev(pos, pl, *np, mode);
+			tpos = get_prev(pos, pl, np, mode);
 			if (tpos >= 0) {
 				pos = tpos;
 				row--;
@@ -2764,13 +2763,13 @@ static void main_window(struct plist *pl, int *np, FILE *f, int reverse,
 			if (getmouse(&mevent) != OK)
 				break;
 			while (row < mevent.y &&
-			       (tpos = get_next(pos, pl, *np, mode, f, reverse, ignore_blanks, just_diff))
+			       (tpos = get_next(pos, pl, np, mode, f, reverse, ignore_blanks, just_diff))
 			       >= 0) {
 				pos = tpos;
 				row++;
 			}
 			while (row > mevent.y &&
-			       (tpos = get_prev(pos, pl, *np, mode)) >= 0) {
+			       (tpos = get_prev(pos, pl, np, mode)) >= 0) {
 				pos = tpos;
 				row--;
 			}
@@ -2790,9 +2789,11 @@ static void main_window(struct plist *pl, int *np, FILE *f, int reverse,
 			} else {
 				int c;
 				if (pl[pos].is_merge)
-					c = merge_window(&pl[pos], NULL, reverse, 0, 0, ignore_blanks, just_diff);
+					c = merge_window(&pl[pos], NULL, reverse, 0, 0,
+					                 ignore_blanks, just_diff, backup);
 				else
-					c = merge_window(&pl[pos], f, reverse, 0, 0, ignore_blanks, just_diff);
+					c = merge_window(&pl[pos], f, reverse, 0, 0,
+					                 ignore_blanks, just_diff, backup);
 				refresh = 2;
 				if (c) {
 					pl[pos].is_merge = 1;
@@ -2822,7 +2823,7 @@ static void main_window(struct plist *pl, int *np, FILE *f, int reverse,
 		case 'q':
 			cnt = 0;
 			any = 0;
-			for (i = 0; i < *np; i++)
+			for (i = 0; i < np; i++)
 				if (pl[i].end && !pl[i].is_merge)
 					cnt++;
 				else if (pl[i].end)
@@ -2843,12 +2844,12 @@ static void main_window(struct plist *pl, int *np, FILE *f, int reverse,
 			if (ans < 0)
 				break;
 			if (ans) {
-				for (i = 0; i < *np; i++) {
+				for (i = 0; i < np; i++) {
 					if (pl[i].end
 					    && !pl[i].is_merge)
 						save_one(f, &pl[i],
 							 reverse,
-							ignore_blanks);
+							 ignore_blanks, backup);
 				}
 			} else
 				cnt = 0;
@@ -2879,7 +2880,7 @@ static void main_window(struct plist *pl, int *np, FILE *f, int reverse,
 				/* Already saved */
 				mesg = "File is already saved.";
 			} else {
-				if (save_one(f, &pl[pos], reverse, ignore_blanks) == 0) {
+				if (save_one(f, &pl[pos], reverse, ignore_blanks, backup) == 0) {
 					pl[pos].is_merge = 1;
 					snprintf(mesg_buf, cols,
 						 "Saved file %s.",
@@ -2900,6 +2901,8 @@ static void main_window(struct plist *pl, int *np, FILE *f, int reverse,
 				mesg = "Cannot restore a folder.";
 			else if (!pl[pos].is_merge)
 				mesg = "File has not been saved, cannot restore.";
+			else if (!backup)
+				mesg = "Backups are disabled, nothing to restore!";
 			else {
 				/* rename foo.porig to foo, and clear is_merge */
 				char *file = pl[pos].file;
@@ -2919,7 +2922,7 @@ static void main_window(struct plist *pl, int *np, FILE *f, int reverse,
 		case 'I': /* Toggle ignoring blanks */
 			ignore_blanks = ignore_blanks ? 0 : IgnoreBlanks;
 			refresh = 2;
-			for (i = 0; i < *np; i++)
+			for (i = 0; i < np; i++)
 				pl[i].calced = 0;
 			break;
 
@@ -3014,7 +3017,7 @@ static void term_init(int doraw)
 
 int vpatch(int argc, char *argv[], int patch, int strip,
 	   int reverse, int replace, char *outfilename,
-	   int selftest, int ignore_blanks)
+	   int selftest, int ignore_blanks, int backup)
 {
 	/* NOTE argv[0] is first arg...
 	 * Behaviour depends on number of args and 'patch'.
@@ -3071,7 +3074,9 @@ int vpatch(int argc, char *argv[], int patch, int strip,
 			fprintf(stderr, "%s: aborting\n", Cmd);
 			exit(2);
 		}
-		main_window(pl, &num_patches, in, reverse, replace, ignore_blanks, just_diff);
+		pl = sort_patches(pl, &num_patches);
+		main_window(pl, num_patches, in, reverse, replace, ignore_blanks,
+		            just_diff, backup);
 		plist_free(pl, num_patches);
 		fclose(in);
 		break;
@@ -3089,7 +3094,9 @@ int vpatch(int argc, char *argv[], int patch, int strip,
 				fprintf(stderr, "%s: aborting\n", Cmd);
 				exit(2);
 			}
-			main_window(pl, &num_patches, f, reverse, replace,ignore_blanks, just_diff);
+			pl = sort_patches(pl, &num_patches);
+			main_window(pl, num_patches, f, reverse, replace,
+			            ignore_blanks, just_diff, backup);
 			plist_free(pl, num_patches);
 		} else if (strlen(argv[0]) > 4 &&
 			 strcmp(argv[0]+strlen(argv[0])-4, ".rej") == 0) {
@@ -3097,34 +3104,34 @@ int vpatch(int argc, char *argv[], int patch, int strip,
 			origname[strlen(origname) - 4] = '\0';
 			show_merge(origname, f, reverse, 0, NULL, NULL,
 				   replace, outfilename,
-				   selftest, ignore_blanks, just_diff);
+				   selftest, ignore_blanks, just_diff, backup);
 		} else
 			show_merge(argv[0], f, reverse, 1, NULL, NULL,
 				   replace, outfilename,
-				   selftest, ignore_blanks, just_diff);
+				   selftest, ignore_blanks, just_diff, backup);
 
 		break;
 	case 2: /* an orig and a diff/.rej  or two files */
 		if (just_diff) {
 			show_merge(NULL, NULL, reverse, 0, argv[0], argv[1],
 				   replace, outfilename,
-				   selftest, ignore_blanks, just_diff);
+				   selftest, ignore_blanks, just_diff, backup);
 			break;
 		}
 		f = fopen(argv[1], "r");
-		check_dir(argv[1], fileno(f));
 		if (!f) {
 			fprintf(stderr, "%s: cannot open %s\n", Cmd, argv[0]);
 			exit(1);
 		}
+		check_dir(argv[1], fileno(f));
 		show_merge(argv[0], f, reverse, 0, NULL, NULL,
 			   replace, outfilename,
-			   selftest, ignore_blanks, just_diff);
+			   selftest, ignore_blanks, just_diff, backup);
 		break;
 	case 3: /* orig, before, after */
 		show_merge(argv[0], NULL, reverse, 0, argv[1], argv[2],
 			   replace, outfilename,
-			   selftest, ignore_blanks, just_diff);
+			   selftest, ignore_blanks, just_diff, backup);
 		break;
 	}
 
